//---------------------------------------------------------------------------
//
//  allio
//
//  Demonstrates using the add/go/get method to efficiently write and read
//  some or all analog and digital I/O on the LabJack U3.
//  Records the time for 1000 iterations and divides by 1000, to allow
//  verification of the basic command/response communication times of the
//  LabJack U3 as documented in Section 3.1 of the U3 User's Guide.
//
//  support@labjack.com
//  Jul 14, 2009
//----------------------------------------------------------------------

program allio;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  lngGetNextIteration: longint;
  ch: char;

  lngIOType: longint;
  lngChannel: longint;
  dblValue: double;
  ValueDIPort: double;
  ValueAIN: array[0..16] of double;

  time: longint;
  i: longint;
  j: longint;
  numIterations: longint;
  numChannels: longint;
  quickSample: longint;
  longSettling: longint;

  dummyDouble: double;
  dummyInt: longint;

begin
  dblValue := 0;
  lngHandle := 0;
  lngIOType := 0;
  lngChannel := 0;
  ValueDIPort := 0;
  time := 0;
  i := 0;
  j := 0;
  numIterations := 10;
  numChannels := 16; //Number of AIN channels, 0-16.
  quickSample := 0; //Set to TRUE for quick AIN sampling.
  longSettling := 1; //Set to TRUE for extra AIN settling time.
  dummyDouble := 0;
  dummyInt := 0;

  //Open the first found LabJack.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure quickSample.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_RESOLUTION, quickSample, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure longSettling.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_SETTLING_TIME, longSettling, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure the necessary lines as analog.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_ANALOG_ENABLE_PORT, 0, (IntPower(2, numChannels) - 1), numChannels);
  ErrorHandler(lngErrorcode, 0);

  //Now an Add/Go/Get block to configure the timers and counters. These
  //are configured on EIO0-EIO3, so if more than 8 analog inputs are
  //enabled then the analog inputs use these lines.
  if numChannels <= 8 then
  begin
    //Set the timer/counter pin offset to 8, which will put the first
    //timer/counter on EIO0.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 8, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Use the default clock source.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc48MHZ, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Enable 2 timers.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Configure Timer0 as 8-bit PWM.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmPWM8, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Set the PWM duty cycle to 50%.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Configure Timer1 as 8-bit PWM.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 1, LJ_tmPWM8, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Set the PWM duty cycle to 50%.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 32768, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Enable Counter0.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Execute the requests.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);
  end;

  //Now add requests that will be processed every iteration of the loop.

  //Add analog input requests.
  for j := 0 to numChannels - 1 do
  begin
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, j, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);
  end;

  //Set DAC0 to 2.5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_DAC, 0, 2.5, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Read CIO digital lines.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_DIGITAL_PORT, 16, 0, 4, 0);
  ErrorHandler(lngErrorcode, 0);

  //Only do the timer/counter stuff if there are less than 8 analog inputs.
  if numChannels <= 8 then
  begin
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_COUNTER, 0, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_COUNTER, 1, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_TIMER, 0, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_TIMER, 1, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Set the PWM duty cycle to 50%.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Set the PWM duty cycle to 50%.
    lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 32768, 0, 0);
    ErrorHandler(lngErrorcode, 0);
  end;

  time := GetTickCount;

  for i := 0 to numIterations - 1 do
  begin

    //Execute the requests.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);

    //Get all the results.  The input measurement results are stored.  All other
    //results are for configuration or output requests so we are just checking
    //whether there was an error.
    lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);
    lngGetNextIteration := 0;
    while lngErrorcode < LJE_MIN_GROUP_ERROR do
    begin
      if lngIOType = LJ_ioGET_AIN then
        ValueAIN[lngChannel] := dblValue;

      if lngIOType = LJ_ioGET_DIGITAL_PORT then
        ValueDIPort := dblValue;

      lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
      if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
        ErrorHandler(lngErrorcode, lngGetNextIteration);

      lngGetNextIteration := lngGetNextIteration + 1;

    end;

  end;

  time := GetTickCount() - time;

  Writeln('Milleseconds per iteration = ' + FloatToStr(time / numIterations));

  Writeln('Digital Input = ' + FloatToStr(ValueDIPort));

  Writeln('AIN readings from last iteration:');
  for j := 0 to numChannels - 1 do
  begin
    Writeln(ValueAIN[j]);
  end;

  ReadLn(ch);
end.
