//---------------------------------------------------------------------------
//
//  u3simple.dlp
//
//  Basic command/response U3 example using the UD driver in Delphi.
//
//  support@labjack.com
//  July 1, 2013
//----------------------------------------------------------------------

program u3simple;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngGetNextIteration: longint;
  dblDriverVersion: double;
  lngIOType, lngChannel: longint;
  dblValue: double;
  Value0, Value1, Value2: double;
  ValueDIBit, ValueDIPort, ValueCounter: double;
  lngHandle: LJ_HANDLE;
  ch: char;
  lngTemp: longint;
  dblTemp: double;
begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  Value0 := 9999;
  Value1 := 9999;
  Value2 := 9999;
  ValueDIBit := 9999;
  ValueDIPort := 9999;
  ValueCounter := 9999;
  lngHandle := 0;
  lngTemp := 0;
  dblTemp := 0;


  //Read and display the UD version.
  dblDriverVersion := GetDriverVersion();
  Writeln('UD Driver Version = ' + FloatToStrF(dblDriverVersion, ffFixed, 4, 3));
  Writeln('');

  //Open the first found LabJack U3.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //First some configuration commands.  These will be done with the ePut
  //function which combines the add/go/get into a single call.

  //Configure FIO0-FIO3 as analog, all else as digital.  That means we
  //will start from channel 0 and update all 16 flexible bits.  We will
  //pass a value of b0000000000001111 or d15.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_ANALOG_ENABLE_PORT, 0, 15, 16);
  ErrorHandler(lngErrorcode, 0);

  //Set the timer/counter pin offset to 7, which will put the first
  //timer/counter on FIO7.
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 7, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable Counter1 (FIO7).
  lngErrorcode := ePut(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 1, 0);
  ErrorHandler(lngErrorcode, 0);


  //The following commands will use the add-go-get method to group
  //multiple requests into a single low-level function.

  //Request a single-ended reading from AIN0.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 0, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a single-ended reading from AIN1.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 1, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request a reading from AIN2 using the Special range.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN_DIFF, 2, 0, 32, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set DAC0 to 3.5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_DAC, 0, 3.5, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set digital output FIO4 to output-high.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_DIGITAL_BIT, 4, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Read digital input FIO5.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_DIGITAL_BIT, 5, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Read digital inputs FIO5 through FIO6.
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_DIGITAL_PORT, 5, 0, 2, 0);
  ErrorHandler(lngErrorcode, 0);

  //Request the value of Counter1 (FIO7).
  lngErrorcode := AddRequest(lngHandle, LJ_ioGET_COUNTER, 1, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  while True do
  begin
    //Execute the requests.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);

    //Get all the results.  The input measurement results are stored.  All other
    //results are for configuration or output requests so we are just checking
    //whether there was an error.
    lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel,
      dblValue, lngTemp, dblTemp);
    ErrorHandler(lngErrorcode, 0);

    lngGetNextIteration := 0;  //Used by the error handling function.

    while lngErrorcode < LJE_MIN_GROUP_ERROR do
    begin
      if lngIOType = LJ_ioGET_AIN then
      begin
        if lngChannel = 0 then
          Value0 := dblValue;
        if lngChannel = 1 then
          Value1 := dblValue;
      end;
      if lngIOType = LJ_ioGET_AIN_DIFF then
        Value2 := dblValue;
      if lngIOType = LJ_ioGET_DIGITAL_BIT then
        ValueDIBit := dblValue;
      if lngIOType = LJ_ioGET_DIGITAL_PORT then
        ValueDIPort := dblValue;
      if lngIOType = LJ_ioGET_COUNTER then
        ValueCounter := dblValue;

      lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, lngTemp, dblTemp);
      if lngErrorcode <> LJE_NO_MORE_DATA_AVAILABLE then
        ErrorHandler(lngErrorcode, lngGetNextIteration);

      lngGetNextIteration := lngGetNextIteration + 1;
    end;

    Writeln('AIN0 = ' + FloatToStr(Value0));
    Writeln('AIN1 = ' + FloatToStr(Value1));
    Writeln('AIN2 = ' + FloatToStr(Value2));
    Writeln('FIO5 = ' + FloatToStr(ValueDIBit));
    Writeln('FIO5-FIO6 = ' + FloatToStr(ValueDIPort)); //Will read 3 (binary 11) if both lines are pulled-high as normal.
    Writeln('Counter1 (FIO7) = ' + FloatToStr(ValueCounter));

    Writeln('');
    Writeln('Press Enter to go again or (q) and then Enter to quit');
    Read(ch);
    if ch = 'q' then
      Break;
  end;
end.
