//---------------------------------------------------------------------------
//
//  thermocouple_ljtia
//
//  Demonstrates using the TCVoltsToTemp to yield temperature readings for
//  a thermocouple connected to a U3 via a LJTick-InAmp
//
//  Setup:
//  TC plus to INA+.
//  TC minus to INA-.
//  10k resistor from INA- to GND on LJTIA (a short is also acceptable in most cases).
//  LJTIA connected to FIO4/FIO5 block on U3.
//  LJTIA offset set to 0.4 volts.
//  LJTIA channel A gain set to 51.
//
//  For the best accuracy, we recommend doing a quick offset calibration as
//  described in the comments below, and using an external cold-junction
//  temperature sensor rather than the internal U3 temp sensor.  The internal
//  temp sensor is often sufficient, but an external sensor, such as
//  the LM34CAZ (national.com) or the EI-1034 (labjack.com), placed near
//  the LJTIA, can provide better CJC, particularly when the U3 itself
//  is subjected to varying temperatures.
//
//  support@labjack.com
//  Jul 15, 2009
//----------------------------------------------------------------------

program twounits;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  lngChannel, lngIOType: longint;
  dblValue, valueAIN: double;  //Analog Voltage Value
  tempChannel: longint;
  tcVolts, cjTempK, pTCTempK: double;
  tcType: longint;
  offsetVoltage: double;
  running: boolean;
  temp: string;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngHandle := 0;
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  valueAIN := 0;
  tempChannel := 4; //Channel which the TC/LJTIA is on (FIO4).
  tcVolts := 0;
  cjTempK := 0;
  pTCTempK := 0;

  tcType := LJ_ttK;
  //Set the temperature sensor to a k type thermocouple
  //Possible Thermocouple types are:
  //LJ_ttB = 6001
  //LJ_ttE = 6002
  //LJ_ttJ = 6003
  //LJ_ttK = 6004
  //LJ_ttN = 6005
  //LJ_ttR = 6006
  //LJ_ttS = 6007
  //LJ_ttT = 6008


  //Offset calibration:  The nominal voltage offset of the LJTick is
  //0.4 volts.  For improved accuracy, though, you should measure the
  //overall system offset.  We know that if the end of the TC is at the
  //same temperature as the cold junction, the voltage should be zero.
  //Put the end of the TC near the LJTIA to make sure they are at the same
  //temperature, and note the voltage measured by FIO4.  This is the actual
  //offset that can be entered below.
  offsetVoltage := 0.4;

  //Open the first found LabJack U3 via USB.
  lngErrorcode := OpenLabJack(LJ_dtU3, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Reset all U3 ports to their default configuration.
  //This is recommended prior to using the U3 to ensure the U3's ports are
  //in the proper state.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  lngErrorcode := ePut(lngHandle, LJ_ioPUT_ANALOG_ENABLE_BIT, tempChannel, 1, 0);
  ErrorHandler(lngErrorcode, 0);

  // Read temperature until user stops it
  running := True;
  while running do
  begin

    lngErrorcode := 0;
    lngIOType := 0;
    lngChannel := 0;
    tcVolts := 0;
    cjTempK := 0;
    pTCTempK := 0;

    //Add analog input requests.
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, tempChannel, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Add request for internal temperature reading -- Internal temp sensor uses
    //analog input channel 30 on the U3.
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 30, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Execute all requests on the labjack lngHandle.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);

    //Get all the results.  The first result should be the voltage reading of the
    //temperature channel.
    lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel,
      valueAIN, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);

    //The second result should be the internal temp sensor.
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel,
      cjTempK, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);


    //Display Voltage Reading
    Writeln('Analog ', tempChannel, ' : ', valueAIN);

    //Display the internal temperature sensor reading.  This example uses
    //that value for cold junction compensation.
    Writeln('U3 internal sensor : ', cjTempK, ' deg K');

    //To get the thermocouple voltage we subtract the offset from the AIN
    //voltage and divide by the LJTIA gain.
    tcVolts := (valueAIN - offsetVoltage) / 51;

    //Convert TC voltage to temperature.
    lngErrorcode := TCVoltsToTemp(tcType, tcVolts, cjTempK, pTCTempK);

    //Display Temperature
    Writeln('Thermocouple sensor: ', pTCTempK, ' deg K');

    Sleep(1500);

    Writeln('Press q to quit or enter to go again');

    ReadLn(temp);
    if temp = 'q' then
      running := False;

  end;
end.
