//  Demonstrates using the TCVoltsToTemp function to yield temperature
//  readings for a thermocouple directly connected to a U6.
//
//  Setup:
//  TC plus to AIN0.
//  TC minus to GND.
//
//  If you do not want all the thermocouple minus leads tied together
//  at GND, you can use differential channels instead.  Connect the minus
//  to AIN1 in that case, and use a 10k resistor from AIN1 to GND to
//  provide a path for bias currents.
//
//  support@labjack.com
//  December 29, 2009

program twounits;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  lngChannel, lngIOType: longint;
  dblValue, valueAIN: double;  //Analog Voltage Value
  time, j: longint;
  tempChannel: longint;
  tcVolts, cjTempK, pTCTempK: double;
  tcType: longint;
  offsetVoltage: double;
  running: boolean;
  temp: string;
  dummyInt: longint;
  dummyDouble: double;
  ainResolution: double;
  dblInternal: double;
  range: double;

begin
  lngHandle := 0;
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  valueAIN := 0;
  time := 0;
  j := 0;
  tempChannel := 0; //Channel which the TC/LJTIA is on (AIN0).
  tcVolts := 0;
  cjTempK := 0;
  pTCTempK := 0;
  ainResolution := 18;
  range := LJ_rgBIPP1V;

  tcType := LJ_ttK;
  //Set the temperature sensor to a k type thermocouple
  //Possible Thermocouple types are:
  //LJ_ttB = 6001
  //LJ_ttE = 6002
  //LJ_ttJ = 6003
  //LJ_ttK = 6004
  //LJ_ttN = 6005
  //LJ_ttR = 6006
  //LJ_ttS = 6007
  //LJ_ttT = 6008


  //Offset calibration:  The nominal voltage offset of the LJTick is
  //0.4 volts.  For improved accuracy, though, you should measure the
  //overall system offset.  We know that if the end of the TC is at the
  //same temperature as the cold junction, the voltage should be zero.
  //Put the end of the TC near the LJTIA to make sure they are at the same
  //temperature, and note the voltage measured by FIO4.  This is the actual
  //offset that can be entered below.
  offsetVoltage := 0.4;

  //Open the first found LabJack U6 via USB.
  lngErrorcode := OpenLabJack(LJ_dtU6, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Configure the desired resolution.
  lngErrorcode := eGet(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_RESOLUTION, ainResolution, 0);
  ErrorHandler(lngErrorcode, 0);

  // Set the range on the ananlog input channel to +/- 0.1 volts (x100 gain)
  lngErrorcode := eGet(lngHandle, LJ_ioPUT_AIN_RANGE, lngChannel, range, 0);
  ErrorHandler(lngErrorcode, 0);

  Writeln('Press any key to quit\n');

  // Read temperature until user stops it
  running := True;
  while running do
  begin

    lngErrorcode := 0;
    lngIOType := 0;
    lngChannel := 0;
    time := 0;
    tcVolts := 0;
    cjTempK := 0;
    pTCTempK := 0;

    //Add analog input requests.
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, tempChannel, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Add request for internal temperature reading -- Internal temp sensor uses
    //analog input channel 14 on the U6.
    lngErrorcode := AddRequest(lngHandle, LJ_ioGET_AIN, 14, 0, 0, 0);
    ErrorHandler(lngErrorcode, 0);

    //Execute all requests on the labjack lngHandle.
    lngErrorcode := GoOne(lngHandle);
    ErrorHandler(lngErrorcode, 0);

    //Get all the results.  The first result should be the voltage reading of the
    //temperature channel.
    lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, valueAIN, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);

    //The second result should be the internal temp sensor.
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblInternal, dummyInt, dummyDouble);
    ErrorHandler(lngErrorcode, 0);

    //The cold junction is the screw-terminal block where the thermocouple
    //is connected.  As discussed in the U6 User's Guide, add 2.5 degrees C
    //to the internal temp sensor reading.  If using the CB37 rather than
    //the built-in screw terminals, just add 1.0 degrees C.
    cjTempK := dblInternal + 2.5;

    //Display Voltage Reading
    Writeln('Analog ', tempChannel, ' : ', valueAIN);

    //Display the internal temperature sensor reading.  This example uses
    //that value for cold junction compensation.
    Writeln(Format('U6 internal sensor: %f deg K', [dblInternal]));

    //Convert TC voltage to temperature.
    lngErrorcode := TCVoltsToTemp(tcType, tcVolts, cjTempK, pTCTempK);

    //Display Temperature
    Writeln(Format('Thermocouple sensor: %f deg K', [pTCTempK]));

    Sleep(1500);

    Writeln('Press q to quit or enter to go again');

    ReadLn(temp);
    if temp = 'q' then
      running := False;

  end;
end.
