//---------------------------------------------------------------------------
//
//  ue9_externaltriggerstream.c
//
//  Does a 6-channel externally triggered stream of AIN0, AIN1, EIO_FIO,
//  MIO_CIO, Timer0_Low, and Timer0_High.
//
//  Timer0 (FIO0) is enabled as SystemTimerLow (mode 10).  With an externally
//  triggered stream, the timing is determined by the external trigger signal,
//  so this timer value can be used to timestamp each scan.
//
//  Timer1 (FIO1) is enabled as frequency output at 1 kHz, and can be
//  used as the external trigger signal for testing.
//
//  External triggering is enabled for the stream, so the trigger signal should
//  be connected to FIO2.
//
//  For testing, connect FIO1 to FIO2.
//
//  support@labjack.com
//  July 16, 2009
//----------------------------------------------------------------------


program ue9triggerstream;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngHandle: LJ_HANDLE;
  lngGetNextIteration: longint;
  i, k: longint;
  lngIOType, lngChannel: longint;
  dblValue, dblCommBacklog: double;
  dummyInt: longint;
  dummyDouble: double;

  //The actual scan rate is determined by the external clock, but we need
  //an idea of how fast the scan rate will be so that we can make
  //the buffers big enough.  Also, the driver needs to have an idea of the
  //expected scan rate to help it decide how big of packets to transfer.
  scanRate: double;
  delayms: longint;
  numScans: double;  //2x the expected # of scans (2*scanRate*delayms/1000)
  numScansRequested: double;
  adblData: array[0..11999] of double;  //Max buffer size (#channels*numScansRequested)

begin
  lngHandle := 0;
  i := 0;
  k := 0;
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  dblCommBacklog := 0;
  dblValue := 0;
  dblCommBacklog := 0;
  scanRate := 1000;
  delayms := 1000;
  numScans := 2000;

  //Open the first found LabJack UE9 over USB.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode := OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Make sure the UE9 is not streaming.
  lngErrorcode := eGet(lngHandle, LJ_ioSTOP_STREAM, 0, dummyDouble, dummyInt);
  ErrorHandler(lngErrorcode, 0);


  //Disable all timers and counters to put everything in a known initial state.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);


  //First we will configure Timer0 as system timer low and configure Timer1 to
  //output a 1000 Hz square wave.

  //Use the fixed 750kHz timer clock source.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc750KHZ, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the divisor to 3 so the actual timer clock is 250 kHz.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Enable 2 timers.  They will use FIO0-FIO1.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer0 as system timer low.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmSYSTIMERLOW, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer1 as frequency output.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 1, LJ_tmFREQOUT, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Set the frequency output on Timer1 to 1000 Hz (250000/(2*125) = 1000).
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_VALUE, 1, 125, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests on a single LabJack.  The driver will use a
  //single low-level TimerCounter command to handle all the requests above.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
      ErrorHandler(lngErrorcode, lngGetNextIteration);

    lngGetNextIteration := lngGetNextIteration + 1;
  end;




  //Configure the stream:
  //Configure resolution for all analog inputs.  Since the test external clock
  //is at 1000 Hz, and we are scanning 6 channels, we will have a
  //sample rate of 6000 samples/second.  That means the maximum resolution
  //we could use is 13-bit.  We will use 12-bit in this example.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chAIN_RESOLUTION, 12, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure the analog input range on channel 0 for bipolar +-5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_AIN_RANGE, 0, LJ_rgBIP5V, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure the analog input range on channel 1 for bipolar +-5 volts.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_AIN_RANGE, 1, LJ_rgBIP5V, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Give the driver a 5 second buffer (scanRate * 6 channels * 5 seconds).
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_BUFFER_SIZE, scanRate * 6 * 5, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure reads to retrieve whatever data is available without waiting (wait mode LJ_swNONE).
  //See comments below to change this program to use LJ_swSLEEP mode.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_WAIT_MODE, LJ_swNONE, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Configure for external triggering.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSTREAM_EXTERNAL_TRIGGER, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  //Define the scan list.
  lngErrorcode := AddRequest(lngHandle, LJ_ioCLEAR_STREAM_CHANNELS, 0, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 0, 0, 0, 0);  //AIN0
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 1, 0, 0, 0);  //AIN1
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 193, 0, 0, 0);  //EIO_FIO
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 194, 0, 0, 0);  //MIO_CIO
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 200, 0, 0, 0);
  //Timer0 LSW
  ErrorHandler(lngErrorcode, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioADD_STREAM_CHANNEL, 224, 0, 0, 0);
  //Timer0 MSW
  ErrorHandler(lngErrorcode, 0);

  //Execute the list of requests.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
      ErrorHandler(lngErrorcode, lngGetNextIteration);
    lngGetNextIteration := lngGetNextIteration + 1;
  end;

  //Start the stream.
  lngErrorcode := eGet(lngHandle, LJ_ioSTART_STREAM, 0, dblValue, 0);
  ErrorHandler(lngErrorcode, 0);


  //Read data
  Writeln('Press and hold esc to end');
  while (GetKeyState(VK_Escape) = 0) or (GetKeyState(VK_Escape) = 1) do
    //Loop will run until esc key is pressed and held
  begin
    //Since we are using wait mode LJ_swNONE, we will wait a little, then
    //read however much data is available.  Thus this delay will control how
    //fast the program loops and how much data is read each loop.  An
    //alternative common method is to use wait mode LJ_swSLEEP where the
    //stream read waits for a certain number of scans.  In such a case
    //you would not have a delay here, since the stream read will actually
    //control how fast the program loops.

    //To change this program to use sleep mode,
    //  -change numScans to the actual number of scans desired per read,
    //  -change wait mode addrequest value to LJ_swSLEEP,
    //  -comment out the following Sleep command.

    Sleep(delayms);  //Remove if using LJ_swSLEEP.

    //init array so we can easily tell if it has changed
    for k := 0 to Trunc(numScans * 2 - 1) do
      adblData[k] := 9999.0;

    //Read the data.  We will request twice the number we expect, to
    //make sure we get everything that is available.
    //Note that the array we pass must be sized to hold enough SAMPLES, and
    //the Value we pass specifies the number of SCANS to read.
    numScansRequested := numScans;
    lngErrorcode := eGet(lngHandle, LJ_ioGET_STREAM_DATA, LJ_chALL_CHANNELS, numScansRequested, @adblData[0]);
    //This displays the number of scans that were actually read.
    Writeln('');
    Writeln('Iteration # ', i);
    Writeln('Number read = ', numScansRequested);
    //This displays just the first scan.
    Writeln('First scan = ', adblData[0], ', ', adblData[1], ', ', adblData[2], ', ', adblData[3], ', ', adblData[4], ', ', adblData[5]);
    ErrorHandler(lngErrorcode, 0);
    //Retrieve the current Comm backlog.  The UD driver retrieves stream data from
    //the UE9 in the background, but if the computer is too slow for some reason
    //the driver might not be able to read the data as fast as the UE9 is
    //acquiring it, and thus there will be data left over in the UE9 buffer.
    lngErrorcode := eGet(lngHandle, LJ_ioGET_CONFIG, LJ_chSTREAM_BACKLOG_COMM, dblCommBacklog, 0);
    Writeln('Comm Backlog = ', dblCommBacklog);
    i := i + 1;
  end;


  //Stop the stream
  lngErrorcode := eGet(lngHandle, LJ_ioSTOP_STREAM, 0, dummyDouble, dummyInt);
  ErrorHandler(lngErrorcode, 0);

  //Disable the timers.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);

  Writeln('Done');
  Readln;
end.
