//---------------------------------------------------------------------------
//
//  Quadrature
//
//  Enables quadrature input on FIO0/FIO1 and reads the current count
//  every half second, until a key is pressed.  To make something happen
//  without an actual quadrature signal, connect a couple wires to
//  ground and tap them on FIO0 and FIO1.
//
//  support@labjack.com
//  July 16, 2009
//----------------------------------------------------------------------

program ue9quadrature;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

//This is our simple error handling procedure that is called after every UD
//function call.  This function displays the errorcode and string description
//of the error.  It also has an iteration input that is useful when processing
//results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: LJ_ERROR;
  lngGetNextIteration: longint;
  lngIOType, lngChannel: longint;
  dblValue: double;
  lngHandle: LJ_HANDLE;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  lngHandle := 0;

  //Open the LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  //lngErrorcode := OpenLabJack (LJ_dtUE9, LJ_ctETHERNET, '192.168.1.209', 0, lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Disable all timers and counters to put everything in a known initial state.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //First enable the quadrature input.

  //Enable 2 timers for phases A and B.  They will use FIO0 and FIO1.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer0 as quadrature.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmQUAD, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Configure Timer1 as quadrature.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_TIMER_MODE, 1, LJ_tmQUAD, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Execute the requests on a single LabJack.  The driver will use a
  //single low-level TimerCounter command to handle all the requests above.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
      ErrorHandler(lngErrorcode, lngGetNextIteration);

    lngGetNextIteration := lngGetNextIteration + 1;
  end;



  while (GetKeyState(VK_Escape) = 0) or (GetKeyState(VK_Escape) = 1) do
    //Program will run until esc key is held down
  begin

    //Wait 500 milliseconds
    Sleep(500);

    //Request a read from Timer0.  Timer0 and Timer1 both return the same
    //quadrature value.
    lngErrorcode := eGet(lngHandle, LJ_ioGET_TIMER, 0, dblValue, 0);
    ErrorHandler(lngErrorcode, 0);
    Writeln('Quad Counter = ', dblValue);

  end;


  //Disable the timers and the FIO lines will return to digital I/O.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
  lngErrorcode := GoOne(lngHandle);

  Readln;
end.
