//---------------------------------------------------------------------------
//
//  SPI
//
//  Demonstrates SPI communication.
//
//  You can short MOSI to MISO for testing.
//
//  MOSI    FIO0
//  MISO    FIO1
//  CLK     FIO2
//  CS      FIO3
//
//  If you short MISO to MOSI, then you will read back the same bytes that you write.  If you short
//  MISO to GND, then you will read back zeros.  If you short MISO to VS or leave it
//  unconnected, you will read back 255s.
//
//  Tested with UD driver V2.73 and UE9 firmware V1.40.
//
//  support@labjack.com
//  July 16, 2009
//----------------------------------------------------------------------


program spi;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  Math,
  LJUDDelphi in '..\..\..\Wrapper\LJUDDelphi.pas';

  //This is our simple error handling procedure that is called after every UD
  //function call.  This function displays the errorcode and string description
  //of the error.  It also has an iteration input that is useful when processing
  //results in a loop (getfirst/getnext).
  procedure ErrorHandler(lngErrorcode: LJ_ERROR; lngIteration: longint);
  var
    err: array[0..254] of AnsiChar;
  begin
    if lngErrorcode <> LJE_NOERROR then
    begin
      ErrorToString(lngErrorcode, @err);
      Writeln('Error number = ' + IntToStr(lngErrorcode));
      Writeln('Error string = ' + string(err));
      Writeln('Iteration = ' + IntToStr(lngIteration));
      Writeln('');
      if lngErrorcode > LJE_MIN_GROUP_ERROR then
      begin
        //Quit if this is a group error.
        Readln;
        Halt;
      end;
    end;
  end;

var
  lngErrorcode: longint;
  lngGetNextIteration: longint;
  lngIOType, lngChannel: longint;
  dblValue: double;
  lngHandle: LJ_HANDLE;
  numSPIBytesToTransfer: double;
  dataArray: array[0..49] of byte;
  dummyInt: longint;
  dummyDouble: double;

begin
  lngIOType := 0;
  lngChannel := 0;
  dblValue := 0;
  lngHandle := 0;

  //Open the LabJack UE9.
  lngErrorcode := OpenLabJack(LJ_dtUE9, LJ_ctUSB, '1', 1, lngHandle);
  ErrorHandler(lngErrorcode, 0);

  //Start by using the pin_configuration_reset IOType so that all
  //pin assignments are in the factory default condition.
  lngErrorcode := ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //First, configure the SPI communication.

  //Enable automatic chip-select control.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_AUTO_CS, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Do not disable automatic digital i/o direction configuration.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_DISABLE_DIR_CONFIG, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //Mode A:  CPHA=1, CPOL=1.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_MODE, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //125kHz clock.
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_CLOCK_FACTOR, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //MOSI is FIO0
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_MOSI_PINNUM, 0, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //MISO is FIO1
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_MISO_PINNUM, 1, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //CLK is FIO2
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_CLK_PINNUM, 2, 0, 0);
  ErrorHandler(lngErrorcode, 0);

  //CS is FIO3
  lngErrorcode := AddRequest(lngHandle, LJ_ioPUT_CONFIG, LJ_chSPI_CS_PINNUM, 3, 0, 0);
  ErrorHandler(lngErrorcode, 0);


  //Execute the requests on a single LabJack.  The driver will use a
  //single low-level TimerCounter command to handle all the requests above.
  lngErrorcode := GoOne(lngHandle);
  ErrorHandler(lngErrorcode, 0);


  //Get all the results just to check for errors.
  lngErrorcode := GetFirstResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
  ErrorHandler(lngErrorcode, 0);
  lngGetNextIteration := 0;
  while lngErrorcode < LJE_MIN_GROUP_ERROR do
  begin
    lngErrorcode := GetNextResult(lngHandle, lngIOType, lngChannel, dblValue, dummyInt, dummyDouble);
    if not lngErrorcode = LJE_NO_MORE_DATA_AVAILABLE then
    begin
      ErrorHandler(lngErrorcode, lngGetNextIteration);
    end;

    lngGetNextIteration := lngGetNextIteration + 1;
  end;


  //This example transfers 4 test bytes.
  numSPIBytesToTransfer := 4;
  dataArray[0] := 170;
  dataArray[1] := 240;
  dataArray[2] := 170;
  dataArray[3] := 240;

  //Transfer the data.  The write and read is done at the same time.
  lngErrorcode := eGet(lngHandle, LJ_ioSPI_COMMUNICATION, 0, numSPIBytesToTransfer, @dataArray);
  ErrorHandler(lngErrorcode, 0);

  //Display the read data.
  Writeln('dataArray[0] = ', dataArray[0]);
  Writeln('dataArray[1] = ', dataArray[1]);
  Writeln('dataArray[2] = ', dataArray[2]);
  Writeln('dataArray[3] = ', dataArray[3]);


  Readln;
end.
